@@ -157,6 +157,13 @@ drcCifWarning()
  * Side effects:
  *	Updates the DRC technology variables.
  *
+ * Notes:  "centidistance" is by default in centimicrons, but it is really in
+ * whatever units are declared in the cifoutput section;  so if cifoutput
+ * declares nanometers, then units are in nanometers.  This may be in
+ * different units from the non-CIF part of the DRC section.  The escape
+ * sequence substitution in the "Why" text will make sure that the correct
+ * physical dimensions are presented to the end user.
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -194,13 +201,13 @@ drcCifWidth(argc, argv)
     }
 
     scalefactor = drcCifStyle->cs_scaleFactor;
-    centidistance *= drcCifStyle->cs_expander;		// BSI
+    // centidistance *= drcCifStyle->cs_expander;		// BSI
 
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
     drcCifAssign(dpnew, centidistance, dpnext, &CIFSolidBits,
     		&CIFSolidBits, why, centidistance,
-		DRC_FORWARD, thislayer, 0);
+		DRC_FORWARD | DRC_CIFRULE, thislayer, 0);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
     return ((centidistance+scalefactor-1)/scalefactor);
@@ -287,11 +294,11 @@ drcCifSpacing(argc, argv)
     }
 
     scalefactor = drcCifStyle->cs_scaleFactor;
-    centidistance *= drcCifStyle->cs_expander;		// BSI
+    // centidistance *= drcCifStyle->cs_expander;		// BSI
     dpnext = drcCifRules[layer[0]][DRC_CIF_SOLID];
     dpnew = (DRCCookie *) mallocMagic((unsigned) sizeof (DRCCookie));
     drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits,
-    		&cmask, why, centidistance, DRC_FORWARD, layer[1], 0);
+    		&cmask, why, centidistance, DRC_FORWARD | DRC_CIFRULE, layer[1], 0);
     drcCifRules[layer[0]][DRC_CIF_SOLID] = dpnew;
     if (needReverse) dpnew->drcc_flags |= DRC_BOTHCORNERS;
 
@@ -299,7 +306,7 @@ drcCifSpacing(argc, argv)
     dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) sizeof (DRCCookie));
     drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits,
-    		&cmask, why, centidistance, DRC_REVERSE, layer[1], 0);
+    		&cmask, why, centidistance, DRC_REVERSE | DRC_CIFRULE, layer[1], 0);
     drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
 
     if (needReverse)
@@ -311,14 +318,16 @@ drcCifSpacing(argc, argv)
          dpnext = drcCifRules[layer[1]][DRC_CIF_SOLID];
          dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
          drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits, &cmask,
-		why, centidistance, DRC_FORWARD|DRC_BOTHCORNERS, layer[0], 0);
+		why, centidistance, DRC_FORWARD|DRC_BOTHCORNERS | DRC_CIFRULE,
+		layer[0], 0);
          drcCifRules[layer[1]][DRC_CIF_SOLID] = dpnew;
 
 	 // Add rule in reverse direction
 	 dpnext = drcCifRules[layer[1]][DRC_CIF_SPACE];
 	 dpnew = (DRCCookie *) mallocMagic((unsigned) sizeof (DRCCookie));
 	 drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits, &cmask,
-		why, centidistance, DRC_REVERSE|DRC_BOTHCORNERS, layer[0], 0);
+		why, centidistance, DRC_REVERSE|DRC_BOTHCORNERS | DRC_CIFRULE,
+		layer[0], 0);
 	 drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
 
 	 if (layer[0] == layer[1])
@@ -326,14 +335,16 @@ drcCifSpacing(argc, argv)
               dpnext = drcCifRules[layer[1]][DRC_CIF_SPACE];
               dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
               drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits,
-			&cmask, why, centidistance, DRC_REVERSE | DRC_BOTHCORNERS,
+			&cmask, why, centidistance,
+			DRC_REVERSE | DRC_BOTHCORNERS | DRC_CIFRULE,
 			layer[0], 0);
                  drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
 
               dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
               dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
               drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits, &cmask,
-			why, centidistance, DRC_REVERSE | DRC_BOTHCORNERS,
+			why, centidistance,
+			DRC_REVERSE | DRC_BOTHCORNERS | DRC_CIFRULE,
 			layer[1], 0);
                  drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
 	 }
@@ -344,13 +355,13 @@ drcCifSpacing(argc, argv)
          dpnext = drcCifRules[layer[1]][DRC_CIF_SPACE];
          dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
          drcCifAssign(dpnew, scalefactor, dpnext, &DBSpaceBits, &DBZeroTypeBits,
-			why, scalefactor, DRC_FORWARD, layer[0], 0);
+			why, scalefactor, DRC_FORWARD | DRC_CIFRULE, layer[0], 0);
          drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
 
          dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
          dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
          drcCifAssign(dpnew, scalefactor, dpnext, &DBSpaceBits, &DBZeroTypeBits,
-			why, scalefactor, DRC_FORWARD, layer[1], 0);
+			why, scalefactor, DRC_FORWARD | DRC_CIFRULE, layer[1], 0);
          drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
     }
 
@@ -541,7 +552,7 @@ drcCifCheck(arg)
     oldTiles = DRCstatTiles;
 
     CIFGen(arg->dCD_celldef, arg->dCD_celldef, checkRect, CIFPlanes,
-	    &DBAllTypeBits, TRUE, TRUE);
+	    &DBAllTypeBits, TRUE, TRUE, FALSE, (ClientData)NULL);
 
     for (i = 0; i < drcCifStyle->cs_nLayers; i++)
     {
@@ -602,6 +613,14 @@ drcCifTile (tile, arg)
     if (SigInterruptPending) return 1;
     DRCstatTiles++;
 
+    /* For non-Manhattan tiles, if the left side of tile is not the */
+    /* type that is declared by the rule, then skip the left-right  */
+    /* check.							    */
+
+    if (IsSplit(tile))
+	if (SplitSide(tile))
+	    goto tbcheck;
+
     /*
      * Check design rules along a vertical boundary between two tiles.
      *
@@ -790,6 +809,15 @@ drcCifTile (tile, arg)
         }
     }
 
+tbcheck:
+
+    /* For non-Manhattan tiles, if the bottom side of tile is not   */
+    /* the type that is declared by the rule, then skip the top-    */
+    /* bottom check.						    */
+
+    if (IsSplit(tile))
+	if (SplitSide(tile) == SplitDirection(tile))
+	    return 0;
 
     /*
      * Check design rules along a horizontal boundary between two tiles.
@@ -1097,7 +1125,7 @@ drcCifArea(argc, argv)
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
     drcCifAssign(dpnew, centihorizon, dpnext, &CIFSolidBits, &CIFSolidBits,
-		why, centiarea, DRC_AREA | DRC_FORWARD, thislayer, 0);
+		why, centiarea, DRC_AREA | DRC_FORWARD | DRC_CIFRULE, thislayer, 0);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
 
@@ -1163,11 +1191,11 @@ drcCifMaxwidth(argc, argv)
     }
 
     scalefactor = drcCifStyle->cs_scaleFactor;
-    centidistance *= drcCifStyle->cs_expander;		// BSI
+    // centidistance *= drcCifStyle->cs_expander;		// BSI
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
     drcCifAssign(dpnew, centidistance, dpnext, &CIFSolidBits, &CIFSolidBits,
-		why, centidistance, DRC_MAXWIDTH | bend, thislayer, 0);
+		why, centidistance, DRC_MAXWIDTH | DRC_CIFRULE | bend, thislayer, 0);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
     return ((centidistance+scalefactor-1)/scalefactor);
