@@ -3,16 +3,16 @@
  *
  *	Routines to create and manipulate heaps.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -84,7 +84,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 
 extern void heapify();
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -154,7 +154,7 @@ HeapInitType(heap, size, descending, stringIds, keyType)
 		* sizeof (HeapEntry)));
     ASSERT(heap->he_list != NULL, "Malloc failed in HeapInit");
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -190,7 +190,7 @@ HeapKill(heap, func)
     freeMagic((char *) heap->he_list);
     heap->he_list = NULL;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -216,7 +216,7 @@ HeapFreeIdFunc(heap, i)
     if (heap->he_stringId)
 	freeMagic(heap->he_list[i].he_id);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -256,7 +256,7 @@ HeapRemoveTop(heap, entry)
     heapify(heap, 1);
     return entry;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -268,7 +268,7 @@ HeapRemoveTop(heap, entry)
  *	Pointer to the top heap element, or NULL if heap is empty.
  *
  * Side effects:
- *	If the heap has been touched, restore the heap property before 
+ *	If the heap has been touched, restore the heap property before
  *      returning the top element.
  *
  * ----------------------------------------------------------------------------
@@ -290,7 +290,7 @@ HeapLookAtTop(heap)
     heap->he_built = heap->he_used;
     return &heap->he_list[1];
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -348,7 +348,7 @@ heapify(heap, root)
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -417,11 +417,12 @@ HeapAdd(heap, pKey, id)
 	    {
 		/* If odd then new entry is the right half of a pair */
 		cmp = i;
-		if (i & 1)
+		if ((i & 1) && (i != 1))
 		    KEY_LE_COND(keyType, list, i, i-1, cmp = i-1);
 
 		/* Find parent.  If 0 then at the root so quit */
 		if ((i >>= 1) == 0) return;
+
 		KEY_LE_COND(keyType, list, cmp, i, return);
 		list[0] = list[cmp]; list[cmp] = list[i]; list[i] = list[0];
 		heapify(heap, cmp);
@@ -434,11 +435,12 @@ HeapAdd(heap, pKey, id)
 	    {
 		/* If odd then new entry is the right half of a pair */
 		cmp = i;
-		if (i & 1)
+		if ((i & 1) && (i != 1))
 		    KEY_GE_COND(keyType, list, i, i-1, cmp = i-1);
 
 		/* Find parent.  If 0 then at the root so quit */
 		if ((i >>= 1) == 0) return;
+
 		KEY_GE_COND(keyType, list, cmp, i, return);
 		list[0] = list[cmp]; list[cmp] = list[i]; list[i] = list[0];
 		heapify(heap, cmp);
@@ -487,7 +489,7 @@ HeapAddDLong(heap, data, id)
 void
 HeapAddFloat(heap, data, id)
     Heap *heap;
-    float data; 
+    float data;
     char *id;
 {
     union heUnion pKey;
@@ -512,7 +514,7 @@ HeapAddDouble(heap, data, id)
     HeapAdd(heap, &pKey, id);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -545,12 +547,12 @@ HeapDump(heap)
 	    case HE_INT:	printf("%d", heap->he_list[i].he_int); break;
 	    case HE_FLOAT:	printf("%f", heap->he_list[i].he_float); break;
 	    case HE_DOUBLE:	printf("%f", heap->he_list[i].he_double); break;
-	    case HE_DLONG:	printf("%lld", heap->he_list[i].he_dlong); break;
+	    case HE_DLONG:	printf("%"DLONG_PREFIX"d", heap->he_list[i].he_dlong); break;
 	}
 	if (heap->he_stringId == TRUE)
 	    printf("//id %s; ", heap->he_list[i].he_id);
 	else
-	    printf("//id %x; ", heap->he_list[i].he_id);
+	    printf("//id %p; ", heap->he_list[i].he_id);
     }
     printf("\n");
 }
